@@ -4,6 +4,8 @@ import shutil
 import yaml
 
 
+DEFAULT_EXP_SERIES_DIR = 'experiments'
+DEFAULT_EXP_NAME = 'unnamed-experiment'
 # TODO: move error msgs into separate file?
 PATH_NOT_EXISTS_ERROR_MSG = ("`{}` directory or file does not exist")
 
@@ -44,6 +46,9 @@ def check_that_path_does_not_exist(path):
 
 
 def infer_new_experiment_version(experiment_parent_dir: str, prefix: str) -> int:
+    if not os.path.exists(experiment_parent_dir):
+        return 1
+
     experiments = os.listdir(experiment_parent_dir)
     experiments = [exp for exp in experiments if exp.startswith(prefix)]
     versions = [exp[len(prefix) + 1:] for exp in experiments]
@@ -54,3 +59,16 @@ def infer_new_experiment_version(experiment_parent_dir: str, prefix: str) -> int
         return max(versions) + 1
     else:
         return 1
+
+
+def infer_new_experiment_path(experiment_dir: str=None, exp_series_dir: str=None, exp_name: str=None) -> str:
+    if not experiment_dir is None:
+        return experiment_dir
+
+    exp_series_dir = DEFAULT_EXP_SERIES_DIR if exp_series_dir is None else exp_series_dir
+    exp_base_name = DEFAULT_EXP_NAME if exp_name is None else exp_name
+
+    exp_version = infer_new_experiment_version(exp_series_dir, exp_base_name)
+    experiment_dir = os.path.join(exp_series_dir, f'{exp_base_name}-{exp_version:05d}')
+
+    return experiment_dir
